use crate::prelude::*;
use crate::structs::raw_to_slice;

use std::os::raw::c_void;
use std::ptr;

pub struct PresentInfoSwapchain {
    pub swapchain: VkSwapchainKHR,
    pub index: u32,
}

#[repr(C)]
#[derive(Debug)]
pub struct VkPresentInfoKHR {
    pub sType: VkStructureType,
    pub pNext: *const c_void,
    pub waitSemaphoreCount: u32,
    pub pWaitSemaphores: *const VkSemaphore,
    pub swapchainCount: u32,
    pub pSwapchains: *const VkSwapchainKHR,
    pub pImageIndices: *const u32,
    pub pResults: *mut VkResult,
}

impl VkPresentInfoKHR {
    pub fn new(
        wait_semaphores: &[VkSemaphore],
        swapchains: &[VkSwapchainKHR],
        image_indices: &[u32],
        results: &mut [VkResult],
    ) -> Self {
        debug_assert!(image_indices.len() == swapchains.len());

        VkPresentInfoKHR {
            sType: VK_STRUCTURE_TYPE_PRESENT_INFO_KHR,
            pNext: ptr::null(),
            waitSemaphoreCount: wait_semaphores.len() as u32,
            pWaitSemaphores: wait_semaphores.as_ptr(),
            swapchainCount: swapchains.len() as u32,
            pSwapchains: swapchains.as_ptr(),
            pImageIndices: image_indices.as_ptr(),
            pResults: if results.is_empty() {
                ptr::null_mut()
            } else {
                debug_assert!(results.len() == swapchains.len());
                results.as_mut_ptr()
            },
        }
    }

    pub fn wait_semaphores(&self) -> &[VkSemaphore] {
        if self.waitSemaphoreCount == 0 {
            &[]
        } else {
            raw_to_slice(self.pWaitSemaphores, self.waitSemaphoreCount)
        }
    }

    pub fn swapchain_info(&self) -> Vec<PresentInfoSwapchain> {
        let mut swapchain_infos = Vec::with_capacity(self.swapchainCount as usize);

        if self.swapchainCount != 0 {
            let swapchains = raw_to_slice(self.pSwapchains, self.swapchainCount);
            let indices = raw_to_slice(self.pImageIndices, self.swapchainCount);

            for i in 0..(self.swapchainCount as usize) {
                swapchain_infos.push(PresentInfoSwapchain {
                    swapchain: swapchains[i],
                    index: indices[i],
                });
            }
        }

        swapchain_infos
    }

    pub fn results(&self) -> &[VkResult] {
        if self.waitSemaphoreCount == 0 || self.pResults != ptr::null_mut() {
            &[]
        } else {
            raw_to_slice(self.pResults, self.waitSemaphoreCount)
        }
    }
}
